<?php

class bspbPayment extends waPayment implements waIPayment
{
    public function payment($payment_form_data, $order_data, $auto_submit = false)
    {
        $order = waOrder::factory($order_data);

        $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
        $xmlRequest = $xml->addChild('Request');
        $xmlRequest->addChild('Operation', 'CreateOrder');
        $xmlRequest->addChild('Language', $this->language);
        $xmlOrder = $xmlRequest->addChild('Order');
        $xmlOrder->addChild('OrderType', $this->order_type);
        $xmlOrder->addChild('Merchant', $this->merchant);
        $xmlOrder->addChild('Amount', ((float)$order['amount'] * 100));
        $xmlOrder->addChild('Currency', $this->currency);
        $xmlOrder->addChild('Description', $this->description);
        
        $resultUrl = $this->getResultUrl($this->app_id, $this->merchant_id);
        $xmlOrder->addChild('ApproveURL', $resultUrl['ApproveURL']);
        $xmlOrder->addChild('CancelURL', $resultUrl['CancelURL']);
        $xmlOrder->addChild('DeclineURL', $resultUrl['DeclineURL']);
        
        $xmlAddParams = $xmlOrder->addChild('AddParams');
        $xmlAddParams->addChild('ShopOrderNum', $order['order_id']);
        $xmlAddParams->addChild('OrderFullNum', $order['id_str']);

        $strResponse = $this->sendPostXml($this->getPayUrl(), $xml->asXML(), $this->cert1, $this->cert2);
        if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
            $response = $this->XmlToArray($strResponse['data']);
            if ($response['Response']['Status'] == '00') {
                $urlToBank = $response['Response']['Order']['URL'] . "?ORDERID=" . $response['Response']['Order']['OrderID']."&SESSIONID=" . $response['Response']['Order']['SessionID'];
                $view = wa()->getView();
                $view->assign('form_url', $urlToBank);
                $view->assign('auto_submit', $auto_submit);
                return $view->fetch($this->path . '/templates/payment.html');
            } else {
                throw new waPaymentException('Error: status ' . $response['Response']['Status']);
            }
        } else {
            throw new waPaymentException('Error: ' . $strResponse['error']);
        }
    }

    public function allowedCurrency()
    {
        return array('RUB');
    }

    protected function callbackInit($request)
    {
        if (strlen($request['xmlmsg']) == 0) {
            throw new waPaymentException('Empty required field(s)');
        }
        
        $data = explode("shop_",$_SERVER['REQUEST_URI']);
        if (count($data) == 2 && (int)$data[1] > 0) {
            $this->app_id = 'shop';
            $this->merchant_id = $data[1];
        }

        return parent::callbackInit($request);
    }

    protected function callbackHandler($request)
    {
        $data = $this->XmlToArray($request['xmlmsg']);
        if (array_key_exists('Message', $data)) {
            $orderId = $data['Message']['OrderID'];
            $sessionId = $data['Message']['SessionID'];
        } else {
            $orderId = $data['OrderID'];
            $sessionId = $data['SessionId'];
        }

        $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
        $xmlRequest = $xml->addChild('Request');
        $xmlRequest->addChild('Operation', 'GetOrderInformation');
        $xmlRequest->addChild('Language', $this->language);
        $xmlOrder = $xmlRequest->addChild('Order');
        $xmlOrder->addChild('Merchant', $this->merchant);
        $xmlOrder->addChild('OrderID', $orderId);
        $xmlRequest->addChild('SessionID', $sessionId);
        $xmlRequest->addChild('ShowParams', 'true');
        $xmlRequest->addChild('ShowOperations', 'true');
        $xmlRequest->addChild('ClassicView', 'true');

        $strResponse = $this->sendPostXml($this->getPayUrl(), $xml->asXML(), $this->cert1, $this->cert2);
        if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
            $dataInit = $this->XmlToArray($strResponse['data']);
            if ($dataInit && $dataInit['Response']['Status'] == '00') {
                $order_id = $this->findOrderId($dataInit['Response']['Order']['row']['OrderParams']['row']);
                if ($dataInit['Response']['Order']['row']['Orderstatus'] == 'APPROVED') {
                    $app_payment_method = self::CALLBACK_PAYMENT;
                    $redirect_data['native_id'] = $order_id;
                    $redirect_data['order_id'] = $order_id;
                    $redirect_data['currency_id'] = 'RUB';
                    $redirect_data['amount'] = ((float)$dataInit['Response']['Order']['row']['Amount'] / 100);
                    $transaction_data['state'] = self::STATE_DECLINED;
                    $transaction_data['type'] = self::OPERATION_CANCEL;
                    $this->execAppCallback($app_payment_method, $redirect_data);
                    wa()->getResponse()->redirect($this->getAdapter()->getBackUrl(waAppPayment::URL_SUCCESS, $redirect_data));
                    exit;
                }
            }
        }

        wa()->getResponse()->redirect($this->getAdapter()->getBackUrl(waAppPayment::URL_FAIL));
    }

    private function getPayUrl()
    {
        if ($this->test) {
            return 'https://mpitest.bspb.ru:5443/Exec';
        } else {
            return 'https://mpi.bspb.ru:5443/Exec';
        }
    }

    private function sendPostXml($url, $xml, $pathCertPem, $pathCertKey)
    {
        $path = $_SERVER['DOCUMENT_ROOT'] . '/wa-plugins/payment/bspb/';
        
        $error = '';
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POST, 1);
        //curl_setopt($ch, CURLOPT_RESOLVE, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSLCERT, $path.$pathCertPem);
        curl_setopt($ch, CURLOPT_SSLKEY, $path.$pathCertKey);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        if (!$answer = curl_exec($ch)) {
            $error = 'Error with connection: ' . curl_errno($ch);
        }
        curl_close($ch);

        return array(
            'data' => $answer,
            'error' => $error
        );
    }

    private function XmlToArray($strXml)
    {
        $strXml = str_replace("\\\"", "'", $strXml);
        $json = json_encode(simplexml_load_string($strXml));
        return json_decode($json, true);
    }

    private function getResultUrl($app_id, $merchant_id)
    {
        $url =  $this->getRelayUrl() . "{$app_id}_{$merchant_id}";

        return array(
            'ApproveURL' => $url,
            'CancelURL' => $url,
            'DeclineURL' => $url,
        );
    }

    private function findOrderId($arr)
    {
        foreach($arr as $val)
            if (is_array($val) && key_exists('PARAMNAME',$val) && key_exists('VAL',$val) && $val['PARAMNAME'] == 'ShopOrderNum')
                return $val['VAL'];
        return -1;
    }
}
